@@ -5,13 +5,7 @@ def configuration(parent_package='', top_path=None):
 
     config.add_subpackage('analysis')
     config.add_subpackage('analysis.dense')
-
     config.add_subpackage('estimation')
-
     config.add_subpackage('flux')
 
-    config.add_extension('kahandot',
-                         sources=['kahandot/kahandot_module.cpp'],
-                         language='c++')
-
     return config
