@@ -24,7 +24,7 @@ n_evals = 50
 final_time = h * n_steps * (n_evals-1)
 
 ode = custom_ode(dim=1, rhs=lambda x: [-.5 * x[0]], h=h, n_steps=n_steps)
-traj = ode.trajectory([[1.]], n_evaluations=n_evals)
+traj = ode.trajectory(x0=1., length=n_evals)
 
 xs = np.linspace(0, final_time, num=n_evals)
 plt.plot(xs, traj, 'x')
