@@ -4,16 +4,18 @@ import warnings
 
 import numpy as np
 import pytest
+from numpy.testing import assert_equal, assert_
 from sklearn.datasets import make_blobs
 from sklearn.utils.extmath import row_norms
 
 import deeptime as dt
-
-import deeptime.clustering._clustering_bindings as bindings
+import deeptime.clustering
+from tests.testing_utilities import ProgressMock
 
 
 def cluster_kmeans(data, k, max_iter=5, init_strategy='kmeans++', fixed_seed=False, n_jobs=None, cluster_centers=None,
-                   callback_init_centers=None, callback_loop=None) -> (dt.clustering.KMeans, dt.clustering.ClusterModel):
+                   callback_init_centers=None, callback_loop=None) -> (
+dt.clustering.KMeans, dt.clustering.ClusterModel):
     est = dt.clustering.KMeans(n_clusters=k, max_iter=max_iter, init_strategy=init_strategy,
                                fixed_seed=fixed_seed, n_jobs=n_jobs,
                                initial_centers=np.array(cluster_centers) if cluster_centers is not None else None)
@@ -22,6 +24,15 @@ def cluster_kmeans(data, k, max_iter=5, init_strategy='kmeans++', fixed_seed=Fal
     return est, model
 
 
+@pytest.mark.parametrize("est", [dt.clustering.KMeans, dt.clustering.MiniBatchKMeans])
+def test_1d_data(est):
+    data = np.ones((500,), dtype=np.float32)
+    estimator = est(1)
+    clustering = estimator.fit(data, initial_centers=np.array([[.5]])).fetch_model()
+    assert_equal(clustering.n_clusters, 1)
+    assert_equal(clustering.cluster_centers[0], [1])
+
+
 @pytest.mark.parametrize("squared", [True, False], ids=lambda x: "squared {}".format(x))
 @pytest.mark.parametrize("precomputed_XX", [True, False], ids=lambda x: "XX {}".format(x))
 @pytest.mark.parametrize("precomputed_YY", [True, False], ids=lambda x: "YY {}".format(x))
@@ -30,11 +41,11 @@ def test_distances(squared, precomputed_XX, precomputed_YY):
     Y = np.random.uniform(-3, 3, size=(70, 3)).astype(np.float64)
     XX = row_norms(X, squared=True).astype(np.float64) if precomputed_XX else None
     YY = row_norms(Y, squared=True).astype(np.float64) if precomputed_YY else None
-
+    impl = deeptime.clustering.metrics['euclidean']
     if squared:
-        dists = bindings.distances_squared(X, Y, XX=XX, YY=YY)
+        dists = impl.distances_squared(X, Y, XX=XX, YY=YY)
     else:
-        dists = bindings.distances(X, Y, XX=XX, YY=YY)
+        dists = impl.distances(X, Y, XX=XX, YY=YY)
     np.testing.assert_equal(dists.shape, (len(X), len(Y)))
     for i in range(len(X)):
         for j in range(len(Y)):
@@ -307,6 +318,14 @@ class TestKmeans(unittest.TestCase):
         assert init == 3
         assert iter <= 2
 
+    def test_noiter(self):
+        initial_centers = np.array([[0, 0, 0], [1, 1, 1]]).astype(np.float32)
+        X = np.random.rand(100, 3)
+        kmeans, model = cluster_kmeans(X, k=2, cluster_centers=initial_centers, max_iter=0, n_jobs=1)
+
+        np.testing.assert_(model.converged)
+        np.testing.assert_array_equal(initial_centers, model.cluster_centers)
+
 
 class TestKmeansResume(unittest.TestCase):
 
@@ -355,3 +374,41 @@ class TestKmeansResume(unittest.TestCase):
                     found[i] = True
 
         assert np.all(found)
+
+
+@pytest.mark.parametrize("with_progress_bar", [True, False])
+def test_kmeans_progress(with_progress_bar):
+    progress_instances = []
+
+    class ProgressFactory:
+        def __new__(cls, *args, **kwargs):
+            progress = ProgressMock()
+            progress_instances.append(progress)
+            return progress
+
+    n_init_callbacks = 0
+    n_loop_callbacks = 0
+
+    def init_callback(*_):
+        nonlocal n_init_callbacks
+        n_init_callbacks += 1
+
+    def loop_callback(*_):
+        nonlocal n_loop_callbacks
+        n_loop_callbacks += 1
+
+    data = np.random.uniform(-10, 10, size=(100, 5))
+    kmeans = dt.clustering.KMeans(2, max_iter=5)
+    if with_progress_bar:
+        kmeans.progress = ProgressFactory
+    kmeans.fit(data, callback_init_centers=init_callback, callback_loop=loop_callback)
+
+    assert_equal(n_init_callbacks, 2)
+    assert_(0 < n_loop_callbacks <= 10)
+
+    if with_progress_bar:
+        assert_equal(len(progress_instances), 2)
+        assert_equal(progress_instances[0].n_update_calls, 2)
+        assert_equal(progress_instances[0].n_close_calls, 1)
+        assert_equal(progress_instances[1].n_update_calls, n_loop_callbacks)
+        assert_equal(progress_instances[1].n_close_calls, 1)
