@@ -1,17 +1,16 @@
 import unittest
 
-import deeptime.markov.tools
+import deeptime.markov.hmm._hmm_bindings as _bindings
 import numpy as np
 import pytest
-import deeptime.markov.hmm._hmm_bindings as _bindings
+from numpy.testing import assert_raises, assert_equal, assert_array_almost_equal, assert_, assert_array_equal
 
-from deeptime.markov.hmm import init, BayesianHMM
+import deeptime.markov.tools
 from deeptime.data import DoubleWellDiscrete
-from deeptime.markov.hmm import MaximumLikelihoodHMM
-from deeptime.markov.hmm import viterbi, HiddenMarkovModel
-from deeptime.markov.hmm import DiscreteOutputModel
-from deeptime.markov.msm import MarkovStateModel
 from deeptime.markov import count_states
+from deeptime.data import prinz_potential
+from deeptime.markov.hmm import DiscreteOutputModel, MaximumLikelihoodHMM, init, BayesianHMM, viterbi, HiddenMarkovModel
+from deeptime.markov.msm import MarkovStateModel
 from tests.markov.msm.test_mlmsm import estimate_markov_model
 from tests.testing_utilities import assert_array_not_equal
 
@@ -128,7 +127,7 @@ class TestMLHMM(unittest.TestCase):
         cls.hmm_lag10_largest = cls.hmm_lag10.submodel_largest(dtrajs=dtraj)
         cls.msm_lag10 = estimate_markov_model(dtraj, 10, reversible=True)
         initial_hmm_1 = init.discrete.metastable_from_data(dtraj, n_hidden_states=2, lagtime=1)
-        cls.hmm_lag1 = MaximumLikelihoodHMM(initial_hmm_1).fit(dtraj).fetch_model()
+        cls.hmm_lag1 = MaximumLikelihoodHMM(initial_hmm_1, lagtime=1).fit(dtraj).fetch_model()
         cls.hmm_lag1_largest = cls.hmm_lag1.submodel_largest(dtrajs=dtraj)
         cls.msm_lag1 = estimate_markov_model(dtraj, 1, reversible=True)
         cls.dtrajs = dtraj
@@ -497,6 +496,28 @@ class TestMLHMM(unittest.TestCase):
             np.testing.assert_equal(I[i].shape[0], hist[A[i]])
             np.testing.assert_equal(I[i].shape[1], 2)
 
+    def test_transform_to_observed_symbols(self):
+        hmsm = self.hmm_lag10_largest
+        dtraj = np.concatenate((hmsm.observation_symbols_full, [500000]))
+        mapped = hmsm.transform_discrete_trajectories_to_observed_symbols(dtraj)[0]
+        for i in range(len(dtraj)):
+            state = dtraj[i]
+            if state in hmsm.observation_symbols:
+                assert_equal(mapped[i], state)
+            else:
+                assert_equal(mapped[i], -1)
+
+    def test_sample_by_observation_probabilities_out_of_sample(self):
+        hmsm = self.hmm_lag10_largest
+        nsample = 50
+        with assert_raises(ValueError):  # symbols not in obs set
+            hmsm.sample_by_observation_probabilities([0, 1, 2], nsample)
+        # sanity check subset of observation symbols
+        hmsm.sample_by_observation_probabilities(np.arange(66), nsample)
+        dtraj2 = np.concatenate((hmsm.observation_symbols, [10000]))
+        # sanity check too large state in there
+        hmsm.sample_by_observation_probabilities(dtraj2, 50)
+
     def test_sample_by_observation_probabilities(self):
         hmsm = self.hmm_lag10_largest
         nsample = 100
@@ -510,6 +531,35 @@ class TestMLHMM(unittest.TestCase):
             for row in samples:
                 np.testing.assert_equal(row[0], 0)  # right trajectory
 
+    def test_sample_by_observation_probabilities_mapping(self):
+        tmat = np.array([[0.9, .1], [.1, .9]])
+        # hidden states correspond to observable states
+        obs = np.eye(2)
+        hmm = HiddenMarkovModel(tmat, obs)
+        # dtraj halfway-split between states 0 and 1
+        dtrajs = np.repeat([0, 1], 10)
+        samples = hmm.sample_by_observation_probabilities(dtrajs, 10)
+        # test that all trajectory indices are 0 (only 1 traj)
+        np.testing.assert_array_equal(np.unique(np.concatenate(samples)[:, 0]), [0])
+        # test that both hidden states map to correct parts of dtraj
+        np.testing.assert_(np.all(samples[0][:, 1] < 10))
+        np.testing.assert_(np.all(samples[1][:, 1] >= 10))
+
+    def test_sample_by_noncrisp_observation_probabilities_mapping(self):
+        tmat = np.array([[0.9, .1], [.1, .9]])
+        # hidden states correspond to observable states
+        obs = np.array([[.9, .1], [.4, .6]])
+        hmm = HiddenMarkovModel(tmat, obs)
+        # dtraj halfway-split between states 0 and 1
+        dtrajs = np.repeat([0, 1], 10)
+        n_samples = 500000
+        samples = hmm.sample_by_observation_probabilities(dtrajs, n_samples)
+        # test that both hidden states map to correct distributions
+        probs_hidden1 = np.histogram(dtrajs[samples[0][:, 1]], bins=2)[0] / n_samples
+        probs_hidden2 = np.histogram(dtrajs[samples[1][:, 1]], bins=2)[0] / n_samples
+        assert_array_almost_equal(probs_hidden1, [.9, .1], decimal=2)
+        assert_array_almost_equal(probs_hidden2, [.4, .6], decimal=2)
+
     def test_simulate_HMSM(self):
         hmsm = self.hmm_lag10_largest
         N = 400
@@ -551,6 +601,26 @@ class TestMLHMM(unittest.TestCase):
         self.assertEqual(hmm_sub.transition_model.stationary_distribution.shape[0], 2)
         self.assertEqual(hmm_sub.transition_model.transition_matrix.shape, (2, 2))
 
+    def test_submodel_empty_state_mapping(self):
+        dtrajs = [np.array([2, 0, 2, 0, 0, 2, 0, 0, 0, 2, 0, 2, 0, 0, 0, 0, 2, 2, 2, 2, 2, 0, 2, 2, 2])]
+        dtrajs_mapped = [(dtrajs[0]/2).astype(int)]
+
+        n_states, lagtime = 2, 1
+        init_hmm = init.discrete.metastable_from_data(dtrajs, n_states, lagtime)
+        estimator = MaximumLikelihoodHMM(init_hmm, lagtime=lagtime)
+
+        fullmm = estimator.fit_fetch(dtrajs)
+        viterbi_from_full = fullmm.compute_viterbi_paths(dtrajs)
+        submm = fullmm.submodel_populous(dtrajs=dtrajs)
+        viterbi_from_sub = submm.compute_viterbi_paths(dtrajs, map_observations_to_submodel=True)
+
+        msg = 'Viterbi path from trajectory that contains empty states is incorrect'
+        assert_array_equal(viterbi_from_full[0], dtrajs_mapped[0], msg)
+        assert_array_equal(viterbi_from_sub[0], dtrajs_mapped[0], msg + ' with submodeled HMM.')
+
+        with assert_raises(RuntimeError):
+            submm.compute_viterbi_paths(dtrajs, map_observations_to_submodel=False)
+
     def test_separate_states(self):
         dtrajs = [np.array([0, 1, 1, 1, 1, 1, 0, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1]),
                   np.array([2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2]), ]
@@ -560,13 +630,20 @@ class TestMLHMM(unittest.TestCase):
         pobs_zeros = ((0, 1, 2, 2, 2), (0, 0, 1, 2, 3))
         assert np.allclose(hmm.output_probabilities[pobs_zeros], 0)
 
+    def test_submodel_disconnect(self):
+        dtraj = np.array([0, 1, 1, 0, 1, 1, 0, 1, 2, 2, 2, 2, 1, 1, 1, 2, 1, 3, 3, 3, 3, 3, 3])
+        init_hmm = init.discrete.metastable_from_data(dtraj, n_hidden_states=4, lagtime=1)
+        hmm = MaximumLikelihoodHMM(init_hmm, lagtime=1).fit_fetch(dtraj)
+        sub_hmm = hmm.submodel_disconnect(1)
+        np.testing.assert_equal(sub_hmm.transition_model.transition_matrix.shape[0], 3)
+
 
 class TestMLHMMPathologicalCases(unittest.TestCase):
 
     def test_1state(self):
         obs = np.array([0, 0, 0, 0, 0], dtype=int)
         init_hmm = init.discrete.metastable_from_data(obs, n_hidden_states=1, lagtime=1)
-        hmm = MaximumLikelihoodHMM(init_hmm).fit(obs).fetch_model()
+        hmm = MaximumLikelihoodHMM(init_hmm, lagtime=1).fit(obs).fetch_model()
         # hmm = bhmm.estimate_hmm([obs], n_states=1, lag=1, accuracy=1e-6)
         p0_ref = np.array([1.0])
         A_ref = np.array([[1.0]])
@@ -583,7 +660,7 @@ class TestMLHMMPathologicalCases(unittest.TestCase):
     def test_2state_step(self):
         obs = np.array([0, 0, 0, 0, 0, 1, 1, 1, 1], dtype=int)
         init_hmm = init.discrete.metastable_from_data(obs, n_hidden_states=2, lagtime=1)
-        hmm = MaximumLikelihoodHMM(init_hmm, accuracy=1e-6).fit(obs).fetch_model()
+        hmm = MaximumLikelihoodHMM(init_hmm, accuracy=1e-6, lagtime=1).fit(obs).fetch_model()
         p0_ref = np.array([1, 0])
         A_ref = np.array([[0.8, 0.2],
                           [0.0, 1.0]])
@@ -600,7 +677,7 @@ class TestMLHMMPathologicalCases(unittest.TestCase):
     def test_2state_2step(self):
         obs = np.array([0, 1, 0], dtype=int)
         init_hmm = init.discrete.metastable_from_data(obs, n_hidden_states=2, lagtime=1)
-        hmm = MaximumLikelihoodHMM(init_hmm).fit(obs).fetch_model()
+        hmm = MaximumLikelihoodHMM(init_hmm, lagtime=1).fit(obs).fetch_model()
         p0_ref = np.array([1, 0])
         A_ref = np.array([[0.0, 1.0],
                           [1.0, 0.0]])
@@ -613,3 +690,15 @@ class TestMLHMMPathologicalCases(unittest.TestCase):
                or np.allclose(hmm.transition_model.transition_matrix, A_ref[np.ix_(perm, perm)], atol=1e-5)
         assert np.allclose(hmm.output_probabilities, B_ref, atol=1e-5) \
                or np.allclose(hmm.output_probabilities, B_ref[[perm]], atol=1e-5)
+
+
+@pytest.mark.parametrize("dtype", [np.float32, np.float64])
+def test_gaussian_prinz(dtype):
+    system = prinz_potential()
+    trajs = system.trajectory(np.zeros((5, 1)), length=5000).astype(dtype)
+    init_ghmm = init.gaussian.from_data(trajs, 4, reversible=True)
+    ghmm = MaximumLikelihoodHMM(init_ghmm, lagtime=1).fit_fetch(trajs)
+    means = ghmm.output_model.means
+
+    for minimum in system.minima:
+        assert_(np.any(np.abs(means - minimum) < 0.1))
